@@ -118,5 +118,7 @@ GdkDisplay *
 init_external_window_x11_display (GError **error)
 {
   gdk_set_allowed_backends ("x11");
-  return gdk_display_open (NULL);
+  if (gtk_init_check ())
+      return gdk_display_get_default ();
+  return NULL;
 }
