@@ -61,7 +61,6 @@ struct _ScreenCastWidget
 
   ShellIntrospect       *shell_introspect;
   GListModel            *filter_model;
-  ulong                  windows_changed_handler_id;
 
   uint                   selection_changed_timeout_id;
   bool                   allow_multiple;
@@ -86,7 +85,7 @@ create_window_widget (ShellWindow *window)
   GtkWidget *window_image;
   GtkWidget *check_image;
   GtkWidget *row;
-  GIcon *icon = NULL;
+  g_autoptr(GIcon) icon = NULL;
   g_autoptr(GDesktopAppInfo) info = NULL;
 
   info = g_desktop_app_info_new (shell_window_get_app_id (window));
@@ -94,6 +93,8 @@ create_window_widget (ShellWindow *window)
     icon = g_app_info_get_icon (G_APP_INFO (info));
   if (icon == NULL)
     icon = g_themed_icon_new ("application-x-executable");
+  else
+    icon = g_object_ref (icon);
   window_image = gtk_image_new_from_gicon (icon);
   gtk_image_set_pixel_size (GTK_IMAGE (window_image), 32);
 
@@ -458,9 +459,6 @@ screen_cast_widget_finalize (GObject *object)
   g_signal_handler_disconnect (widget->display_state_tracker,
                                widget->monitors_changed_handler_id);
 
-  /* Disconnect windows changed listener */
-  g_clear_signal_handler (&widget->windows_changed_handler_id,
-                          widget->shell_introspect);
   shell_introspect_unref_listeners (widget->shell_introspect);
 
   if (widget->selection_changed_timeout_id > 0)
@@ -595,7 +593,7 @@ screen_cast_widget_init (ScreenCastWidget *widget)
   gtk_list_box_bind_model (GTK_LIST_BOX (widget->window_list),
                            widget->filter_model,
                            (GtkListBoxCreateWidgetFunc) create_window_widget,
-                           widget, NULL);
+                           NULL, NULL);
 
   g_object_connect (widget->window_list,
                     "swapped-signal::row-activated", G_CALLBACK (on_row_activated), widget,
